--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: BLAB1 firmware			    					 							--
--	Module name: BLAB1_RAM_TIMING																--
--	Description : 																					--
-- 	BLAB1 RAM timing firmware																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity CONFIG_PLL is
port ( 
			 xCLK    		: in  std_logic;
			 xCLR_ALL 		: in  std_logic;
			 xSCLK			: out std_logic;
			 xDIN				: out std_logic;
			 xS_EN			: out std_logic;			 
			 xPWR_DWN		: out std_logic;			 
			 xPLL_RST		: out std_logic);	
end CONFIG_PLL;

architecture Behavioral of CONFIG_PLL is
	type STATE_TYPE is ( IDLE, PWR_UP, SERIALIZE, GND_STATE, CAL_HOLD, SYNC); 
	signal STATE			: STATE_TYPE;
--------------------------------------------------------------------------------
	signal DIN	  			: std_logic;
	signal S_EN	  			: std_logic;
	signal PWR_DWN	  		: std_logic;
	signal PLL_RST	  		: std_logic;
	signal RAM_SEL			: std_logic_vector(3 downto 0);
	signal xRAM				: std_logic_vector(31 downto 0);
--------------------------------------------------------------------------------
	signal xRAM0	  		: std_logic_vector(31 downto 0);
	signal xRAM1	  		: std_logic_vector(31 downto 0);
	signal xRAM2	  		: std_logic_vector(31 downto 0);
	signal xRAM3	  		: std_logic_vector(31 downto 0);
	signal xRAM4	  		: std_logic_vector(31 downto 0);
	signal xRAM5	  		: std_logic_vector(31 downto 0);
	signal xRAM6	  		: std_logic_vector(31 downto 0);
	signal xRAM7	  		: std_logic_vector(31 downto 0);
	signal xRAM8	  		: std_logic_vector(31 downto 0);
	signal xLOAD_PROM		: std_logic_vector(31 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	--factory default
--	xRAM0 <= x"8184032" & x"0";
--	xRAM1 <= x"8184030" & x"1";
--	xRAM2 <= x"8186030" & x"2";
--	xRAM3 <= x"EB86030" & x"3";
--	xRAM4 <= x"0186031" & x"4";
--	xRAM5 <= x"101C0BE" & x"5";
--	xRAM6 <= x"04BE19A" & x"6";
--	xRAM7 <= x"BD0037F" & x"7";
--	xRAM8 <= x"80005DD" & x"8";
--------------------------------------------------------------------------------			
	--FEED-thru of REF signal, no VCO clean up	
--	xRAM0 <= "11101001010000000000000000100000";
--	xRAM1 <= "11101001010000000000000000000001";
--	xRAM2 <= "11101001010000000000000000000010";
--	xRAM3 <= "11101001010000000000000000000011";
--	xRAM4 <= "11101001010000000000000000000100";
--	xRAM5 <= "00010000000111000000110001110101";
--	xRAM6 <= "00000100101111110001100110100110";
--	xRAM7 <= "10111101000000000011011111110111";
--	xRAM8 <= "00100000000000000101111111111000";
--------------------------------------------------------------------------------			
	--clean up 60 MHz clock	
	xRAM0 <= "11101011000001100000001100010000";
	xRAM1 <= "11101011000001100000001100000001";
	xRAM2 <= "11101011000001100000001100110010";
	xRAM3 <= "11101011000001100000001100000011";
	xRAM4 <= "11101011000001100000001100000100";
	xRAM5 <= "11111111111111000000110001110101";
	xRAM6 <= "11100110101010010111111110100110";
	xRAM7 <= "10111101000000000011011111110111";
	xRAM8 <= "00100000000000000101111111111000";
----------------------------
-- software configuration --
----------------------------	
--REGISTERS
--0	eb0603df
--1	eb0603ff
--2	eb0603ff
--3	eb0603ef
--4	eb0603ff
--5	ffff8c7f
--6	e7a9ffaf
--7	fd0035ff
--8	ffffffff

--PORTS
--0	bf
--1	e0
--2	cf
--3	f9

--INPUTS
--PRI	60
--SEC	0
--AUX	0

--EXTERNAL COMPONENTS
--C4	
--R4	
--C5		
----------------------------	
	xLOAD_PROM <= x"0000001F";
--------------------------------------------------------------------------------			
	process(xCLK,xCLR_ALL)
	variable i	: integer range 0 to 31;
	begin
		if xCLR_ALL = '1' then
			DIN 		<= '0';
			S_EN 		<= '0';
			PWR_DWN 	<= '1';
			PLL_RST 	<= '1';
			RAM_SEL <= x"F";
			STATE	<= PWR_UP;			
		elsif falling_edge(xCLK) then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when PWR_UP =>
					PWR_DWN 	<= '1';
					PLL_RST 	<= '1';
					STATE	<= CAL_HOLD;	
--------------------------------------------------------------------------------	
				when CAL_HOLD =>
					PWR_DWN 	<= '0';
					PLL_RST 	<= '1';
					STATE	<= IDLE;	
--------------------------------------------------------------------------------	
				when IDLE =>
					DIN 		<= '0';
					S_EN 		<= '0';
					PWR_DWN 	<= '0';
					PLL_RST 	<= '0';
					i := 0;
					if RAM_SEL = x"9" then
						RAM_SEL <= x"F";
						STATE	<= SYNC;
					else
						RAM_SEL <= RAM_SEL + 1;
						STATE	<= SERIALIZE;
					end if;
--------------------------------------------------------------------------------	
				when SERIALIZE =>
					S_EN 	<= '1';
					DIN 	<= xRAM(i);
					if i = 31 then
						STATE	<= IDLE;
					else
						i := i + 1;
					end if;
--------------------------------------------------------------------------------	
				when SYNC =>
					PLL_RST 	<= '1';
					if i = 31 then
						STATE	<= GND_STATE;
					else
						i := i + 1;
					end if;				
--------------------------------------------------------------------------------	
				when GND_STATE =>
					DIN 		<= '0';
					S_EN 		<= '0';
					PWR_DWN 	<= '0';
					PLL_RST 	<= '0';
					RAM_SEL <= x"F";
--------------------------------------------------------------------------------	
				when others =>	STATE<=PWR_UP;																
			end case;
		end if;
	end process;					
--------------------------------------------------------------------------------	
	process(RAM_SEL)
	begin
		if RAM_SEL = x"0" then
			xRAM <= xRAM0;
		elsif RAM_SEL = x"1" then
			xRAM <= xRAM1;
		elsif RAM_SEL = x"2" then
			xRAM <= xRAM2;
		elsif RAM_SEL = x"3" then
			xRAM <= xRAM3;
		elsif RAM_SEL = x"4" then
			xRAM <= xRAM4;
		elsif RAM_SEL = x"5" then
			xRAM <= xRAM5;
		elsif RAM_SEL = x"6" then
			xRAM <= xRAM6;
		elsif RAM_SEL = x"7" then
			xRAM <= xRAM7;
		elsif RAM_SEL = x"8" then
			xRAM <= xRAM8;
		elsif RAM_SEL = x"9" then
			xRAM <= xLOAD_PROM;
		else
			xRAM <= x"00000000";
		end if;
	end process;
--------------------------------------------------------------------------------	
	xBUF_SCLK : BUF
	port map (
		I  => xCLK,
		O  => xSCLK);
--------------------------------------------------------------------------------	
	xBUF_DIN : BUF
	port map (
		I  => DIN,
		O  => xDIN);
--------------------------------------------------------------------------------	
	xBUF_S_EN : BUF
	port map (
		I  => S_EN,
		O  => xS_EN);
--------------------------------------------------------------------------------	
	xBUF_PWR_DWN : BUF
	port map (
		I  => PWR_DWN,
		O  => xPWR_DWN);
--------------------------------------------------------------------------------	
	xBUF_PLL_RST : BUF
	port map (
		I  => PLL_RST,
		O  => xPLL_RST);
--------------------------------------------------------------------------------	
end Behavioral;